/*Lifeshell Project, PI Marco Fellin
Jan 2025
Seismodesk, Seismic activity sensor and alert system
Earthquake Early Warning System
Basic example code, please do not trust 100% electronic devices including this piece of software when lives are at stake.

SCOPE:
Monitoring earth acceleration and alert users when an earthquake hits your house.

USE:
Power ON, a calibration and a self check runs automatically. All LED lit shortly.
If green LED is steady ON, system is running correctly.
Shall vibrations exceed certain threshold, yellow or red LED power on, roughly indicating the acceleration amount.
An alert sounds/vibrate/light, according to the output device(s) connected.

Serial print output can be deactivated.

Hardware: MPU6050 sensor and Arduino NANO (other accelerometers and Arduino can be used)
MPU6050 connections: 5V, GND, SCL on A4, SDA on A5 (SCL and SDA Pins for Arduino Nano, modify for your Arduino)
Green LED on PIN D9 to GND via a 120 (100-200) Ohm resistor
Yellow LED on PIN D10 to GND via a 120 (100-200) Ohm resistor
RED LED on PIN D11 to GND via a 220 (150-250) Ohm resistor
White LED on PIN D3 to GND via a 120 (100-200) Ohm resistor

*/
// Based on the Basic demo for accelerometer readings from Adafruit MPU6050
//https://learn.adafruit.com/mpu6050-6-dof-accelerometer-and-gyro/python-and-circuitpython
//https://forum.arduino.cc/t/how-to-export-data-from-arduino-serial-monitor-to-a-csv-or-txt-file/354651/5

#include <Adafruit_MPU6050.h>
#include <Adafruit_Sensor.h>
#include <Wire.h>


Adafruit_MPU6050 mpu;
int LEDwhite = 3; //led pin
int LEDgreen = 9; //led pin
int LEDyellow = 10;   //led pin
int LEDred = 11;   //led pin
unsigned long eventTime = 0; //variable for storing the time since the seism event was detected

void setup(void) {
  Serial.begin(115200);
  while (!Serial)
    delay(10); // will pause Zero, Leonardo, etc until serial console opens
  Serial.println("Adafruit MPU6050 test!");
pinMode(LEDgreen, OUTPUT); //define LED as output
pinMode(LEDyellow, OUTPUT);
pinMode(LEDred, OUTPUT);
pinMode(LEDwhite, OUTPUT);

//LED welcome and check display, on power up
digitalWrite(LEDgreen, HIGH);
delay(300);
digitalWrite(LEDyellow, HIGH);
delay(300);
digitalWrite(LEDred, HIGH);
delay(300);
digitalWrite(LEDwhite, HIGH);
delay(300);
digitalWrite(LEDwhite, LOW);
delay(300);
digitalWrite(LEDred, LOW);
delay(300);
digitalWrite(LEDyellow, LOW);

  // Try to initialize!
  if (!mpu.begin()) {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }
  }
  Serial.println("MPU6050 Found!");

  mpu.setAccelerometerRange(MPU6050_RANGE_8_G);
  Serial.print("Accelerometer range set to: ");
  switch (mpu.getAccelerometerRange()) {
  case MPU6050_RANGE_2_G:
    Serial.println("+-2G");
    break;
  case MPU6050_RANGE_4_G:
    Serial.println("+-4G");
    break;
  case MPU6050_RANGE_8_G:
    Serial.println("+-8G");
    break;
  case MPU6050_RANGE_16_G:
    Serial.println("+-16G");
    break;
  }
  mpu.setGyroRange(MPU6050_RANGE_500_DEG);
  Serial.print("Gyro range set to: ");
  switch (mpu.getGyroRange()) {
  case MPU6050_RANGE_250_DEG:
    Serial.println("+- 250 deg/s");
    break;
  case MPU6050_RANGE_500_DEG:
    Serial.println("+- 500 deg/s");
    break;
  case MPU6050_RANGE_1000_DEG:
    Serial.println("+- 1000 deg/s");
    break;
  case MPU6050_RANGE_2000_DEG:
    Serial.println("+- 2000 deg/s");
    break;
  }


  mpu.setFilterBandwidth(MPU6050_BAND_21_HZ);
  Serial.print("Filter bandwidth set to: ");
  switch (mpu.getFilterBandwidth()) {
  case MPU6050_BAND_260_HZ:
    Serial.println("260 Hz");
    break;
  case MPU6050_BAND_184_HZ:
    Serial.println("184 Hz");
    break;
  case MPU6050_BAND_94_HZ:
    Serial.println("94 Hz");
    break;
  case MPU6050_BAND_44_HZ:
    Serial.println("44 Hz");
    break;
  case MPU6050_BAND_21_HZ:
    Serial.println("21 Hz");
    break;
  case MPU6050_BAND_10_HZ:
    Serial.println("10 Hz");
    break;
  case MPU6050_BAND_5_HZ:
    Serial.println("5 Hz");
    break;
  }
  Serial.println("");
  delay(100);
}


void loop() {

  /* Get new sensor events with the readings */
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);
  /* Print out the values */
  Serial.print("ms ");
  Serial.print(millis());
  Serial.print(" eventTime ");
  Serial.print(eventTime);
  Serial.print(" Acc X: ");
  Serial.print(a.acceleration.x);
  Serial.print(", Y: ");
  Serial.print(a.acceleration.y);
  Serial.print(", Z: ");
  Serial.print(a.acceleration.z);
  Serial.println(" m/s^2");
/*
  Serial.print("Rotation X: ");//acceleration and temp data avoided in serial printing for clarity
  Serial.print(g.gyro.x);
  Serial.print(", Y: ");
  Serial.print(g.gyro.y);
  Serial.print(", Z: ");
  Serial.print(g.gyro.z);
  Serial.println(" rad/s");
  Serial.print("Temperature: ");
  Serial.print(temp.temperature);
  Serial.println(" degC");
*/

//condition for detecting weak accelerations and signalling with the YELLOW LED. Change parameters according to your own sensor values, threshold should be around 0.5 m/s^2 + and - the calm readings
if (a.acceleration.x < -0.8 || a.acceleration.x > 0.5 || a.acceleration.y < -0.5 || a.acceleration.y > 0.5 || a.acceleration.z < 10 || a.acceleration.z > 11) { 
digitalWrite(LEDyellow, HIGH);
//condition for detecting strong accelerations and signalling with the RED LED. Change parameters according to your own sensor values, threshold should be around 0.5 m/s^2 + and - the calm readings
if (a.acceleration.x < -1.8 || a.acceleration.x > 1.5 || a.acceleration.y < -1.5 || a.acceleration.y > 1.5 || a.acceleration.z < 9 || a.acceleration.z > 12) { 
digitalWrite(LEDred, HIGH);
digitalWrite(LEDwhite, HIGH);
}
eventTime = millis(); //mark the time of the event
}

//function for powering off LED, when calm comes back, adjust the time as you wish
if (millis() - eventTime > 30000) {
digitalWrite(LEDred, LOW);
digitalWrite(LEDyellow, LOW);
digitalWrite(LEDwhite, LOW);
eventTime = 0;
}  

}
